/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.commons.database;

import org.sonar.commons.database.dialect.JdbcDialect;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;
import java.sql.Connection;

public class DatabaseConfiguration {
  public static final int DATABASE_VERSION = 33;

  public final static String DEFAULT_JNDI_NAME = "java:comp/env/jdbc/sonar";
  public final static String DEFAULT_DIALECT = JdbcDialect.DERBY.toString();

  private String url = null;
  private String driverClassName = null;
  private String username = null;
  private String password = null;
  private String datasourceName = null;
  private boolean cacheEnabled = true;
  private int isolationLevel;
  private JdbcDialect jdbcDialect = null;
  private boolean createDatabaseFromHibernateMapping = false;
  private int createDatabaseVersion = -1;

  public DatabaseConfiguration(String url, String driverClassName, String username, String password) {
    this.url = url;
    this.driverClassName = driverClassName;
    this.username = username;
    this.password = password;
    this.isolationLevel = Connection.TRANSACTION_READ_UNCOMMITTED;
  }

  public DatabaseConfiguration(String datasourceName) {
    this.datasourceName = datasourceName;
    this.isolationLevel = Connection.TRANSACTION_READ_UNCOMMITTED;
    autodetectJdbcDialect();
  }

  public DatabaseConfiguration(String datasourceName, JdbcDialect dialect) {
    this.datasourceName = datasourceName;
    this.isolationLevel = Connection.TRANSACTION_READ_UNCOMMITTED;
    this.jdbcDialect = dialect;
  }

  public int getIsolationLevel() {
    return isolationLevel;
  }

  public void setIsolationLevel(int isolationLevel) {
    this.isolationLevel = isolationLevel;
  }

  public boolean isCreateDatabaseFromHibernateMapping() {
    return createDatabaseFromHibernateMapping;
  }

  public void unsetCreateDatabaseFromHibernateMapping() {
    this.createDatabaseFromHibernateMapping = false;
    this.createDatabaseVersion = -1;
  }

  public void setCreateDatabaseFromHibernateMapping(int version) {
    this.createDatabaseFromHibernateMapping = true;
    this.createDatabaseVersion = version;
  }

  public int getCreateDatabaseVersion() {
    return createDatabaseVersion;
  }

  public String getDdlMode() {
    if (isCreateDatabaseFromHibernateMapping()) {
      return "create-drop"; // for unit tests
    }
    return "validate"; // default mode
  }

  public boolean isCacheEnabled() {
    return this.cacheEnabled;
  }

  public void setEnableCache(boolean b) {
    this.cacheEnabled = b;
  }

  public String getDatasourceName() {
    return datasourceName;
  }

  public JdbcDialect getJdbcDialect() {
    return jdbcDialect;
  }

  public void setJdbcDialect(JdbcDialect jdbcDialect) {
    this.jdbcDialect = jdbcDialect;
  }

  public String getUrl() {
    return url;
  }

  public String getDriverClassName() {
    return driverClassName;
  }

  public String getUsername() {
    return username;
  }

  public String getPassword() {
    if (password == null) {
      return "";
    }
    return password;
  }

  public String autodetectJdbcDialect() {
    try {
      InitialContext ctx = new InitialContext();
      DataSource ds= (DataSource) ctx.lookup(getDatasourceName());
      String dialect = JdbcDialect.autodetectJdbcDialect(ds);
      if (dialect!=null) {
        setJdbcDialect(JdbcDialect.fromString(dialect));
        return dialect;
      }

    } catch (NamingException e) {

    }
    return null;
  }

  public int hashCode() {
    if (getDatasourceName() != null) {
      return (getJdbcDialect() + getDatasourceName() + isCacheEnabled() + getIsolationLevel()).hashCode();
    }
    return (getUsername() + getPassword() + getUrl() +
        getDriverClassName() + isCacheEnabled() + getIsolationLevel()).hashCode();
  }

  public String toString() {
    final String separator = " # ";
    if (getDatasourceName() != null) {
      return getDatasourceName() + separator + getJdbcDialect() + separator + isCacheEnabled() + separator + getIsolationLevel();
    }
    return getUrl() + separator + getDriverClassName() + separator + getUsername() +
        separator + getPassword() + separator + isCacheEnabled() + separator + getIsolationLevel();
  }
}