package org.sonar.commons.database.dialect;

import org.hibernate.dialect.DB2Dialect;
import org.hibernate.dialect.HSQLDialect;
import org.hibernate.dialect.PostgreSQLDialect;

import javax.persistence.PersistenceException;
import javax.sql.DataSource;
import java.sql.SQLException;


public enum JdbcDialect {
  DB2, DERBY, HSQLDB, MSSQL, MYSQL, ORACLE, POSTGRE;

  /**
   * System property name to force the dialect class name (com.foo.bar.MyHibernateDialectClassName) to use.
   * Can be useful when the dialect impl autodetected does not work correctly with your database.
   */
  public final static String HIBERNATE_JDBC_DIALECT_IMPL = "org.codehaus.sonar.jdbc.dialect.impl";
  /**
   * System property name to force the dialect name (one value of {@link ch.hortis.sonar.model.JdbcData.JDBCDialect}) to use.
   * Can be useful when the dialect name cannot be auto detected from the provided JDBC connection URL
   */
  public final static String HIBERNATE_JDBC_DIALECT_NAME = "org.codehaus.sonar.jdbc.dialect.name";

  public static JdbcDialect fromString(String dialect) throws IllegalArgumentException {
    for (JdbcDialect dialectEnum : JdbcDialect.values()) {
      if (dialectEnum.toString().equalsIgnoreCase(dialect)) {
        return dialectEnum;
      }
    }
    throw new IllegalArgumentException("Unable to map dialect " + dialect + " to one of the JDBCDialect enumeration value");
  }

  public static String getHibernateDialect(JdbcDialect dialect) {
    String dialectImpl = getHibernateDialectImpl();
    if (dialectImpl != null) {
      return dialectImpl;
    }
    if (dialect == JdbcDialect.DB2) {
      return DB2Dialect.class.getName();
    }
    if (dialect == JdbcDialect.DERBY) {
      return DerbyWithDecimalDialect.class.getName();
    }
    if (dialect == JdbcDialect.HSQLDB) {
      return HSQLDialect.class.getName();
    }
    if (dialect == JdbcDialect.MSSQL) {
      return MsSqlDialect.class.getName();
    }
    if (dialect == JdbcDialect.MYSQL) {
      return MySqlWithDecimalDialect.class.getName();
    }
    if (dialect == JdbcDialect.ORACLE) {
      return Oracle10gWithDecimalDialect.class.getName();
    }
    if (dialect == JdbcDialect.POSTGRE) {
      return PostgreSQLDialect.class.getName();
    }
    throw new PersistenceException("Unsupported database for Dialect " + dialect);
  }

  /**
   * Hibernate dialect -> supported db mapping
   *
   * @param url the JDBC URL
   * @return a string containing the hibernate dialect class name to use
   * @throws PersistenceException if the given JDBC URL is not supported
   */
  public static String getHibernateDialect(String url) {
    if (url.toLowerCase().startsWith("jdbc:db2:")) {
      return getHibernateDialect(JdbcDialect.DB2);
    }
    if (url.toLowerCase().startsWith("jdbc:derby:")) {
      return getHibernateDialect(JdbcDialect.DERBY);
    }
    if (url.toLowerCase().startsWith("jdbc:hsqldb:")) {
      return getHibernateDialect(JdbcDialect.HSQLDB);
    }
    if (url.toLowerCase().startsWith("jdbc:microsoft:sqlserver:") ||
        url.toLowerCase().startsWith("jdbc:jtds:sqlserver:")) {
      return getHibernateDialect(JdbcDialect.MSSQL);
    }
    if (url.toLowerCase().startsWith("jdbc:mysql:")) {
      return getHibernateDialect(JdbcDialect.MYSQL);
    }
    if (url.toLowerCase().startsWith("jdbc:oracle:")) {
      return getHibernateDialect(JdbcDialect.ORACLE);
    }
    if (url.toLowerCase().startsWith("jdbc:postgresql:")) {
      return getHibernateDialect(JdbcDialect.POSTGRE);
    }
    if (getHibernateDialectImpl() != null) {
      return getHibernateDialectImpl();
    }
    throw new PersistenceException("Unsupported database for URL " + url +
        " try to set the " + HIBERNATE_JDBC_DIALECT_IMPL +
        " system property with the corresponding hibernate dialect class name");
  }

  /**
   * JDBC URL -> JDBC Dialect
   *
   * @param url the JDBC URL
   * @return a string containing the JDBC dialect identifier to use
   * @throws PersistenceException if the given JDBC URL is not supported
   */
  public static String getHibernateDialectName(String jdbcUrl) {
    String dialectName = getHibernateDialectName();
    if (dialectName != null) {
      return dialectName;
    } else if (jdbcUrl.toLowerCase().startsWith("jdbc:db2:")) {
      return JdbcDialect.DB2.name();
    } else if (jdbcUrl.toLowerCase().startsWith("jdbc:derby:")) {
      return JdbcDialect.DERBY.name();
    } else if (jdbcUrl.toLowerCase().startsWith("jdbc:hsqldb:")) {
      return JdbcDialect.HSQLDB.name();
    } else if (jdbcUrl.toLowerCase().startsWith("jdbc:microsoft:sqlserver:") ||
        jdbcUrl.toLowerCase().startsWith("jdbc:jtds:sqlserver:")) {
      return JdbcDialect.MSSQL.name();
    } else if (jdbcUrl.toLowerCase().startsWith("jdbc:mysql:")) {
      return JdbcDialect.MYSQL.name();
    } else if (jdbcUrl.toLowerCase().startsWith("jdbc:oracle:")) {
      return JdbcDialect.ORACLE.name();
    } else if (jdbcUrl.toLowerCase().startsWith("jdbc:postgresql:")) {
      return JdbcDialect.POSTGRE.name();
    }
    throw new PersistenceException("Unsupported database dialect for URL " + jdbcUrl +
        " try to set the " + HIBERNATE_JDBC_DIALECT_NAME +
        " system property with the corresponding hibernate dialect name");
  }

  public static String getHibernateDialectImpl() {
    String jdbcDialectImpl = System.getProperty(HIBERNATE_JDBC_DIALECT_IMPL);
    if (jdbcDialectImpl != null) {
      try {
        JdbcDialect.class.getClassLoader().loadClass(jdbcDialectImpl);
      } catch (ClassNotFoundException e) {
        throw new PersistenceException("Hibernate dialect class impl " + jdbcDialectImpl + " not found", e);
      }
    }
    return jdbcDialectImpl;
  }

  public static String getHibernateDialectName() {
    String jdbcDialectName = System.getProperty(HIBERNATE_JDBC_DIALECT_NAME);
    if (jdbcDialectName != null) {
      jdbcDialectName = jdbcDialectName.toUpperCase();
      JdbcDialect[] knownDialects = JdbcDialect.values();
      boolean supportedDialect = false;
      for (JdbcDialect dialect : knownDialects) {
        if (dialect.name().equals(jdbcDialectName)) {
          supportedDialect = true;
          break;
        }
      }
      if (!supportedDialect) {
        throw new PersistenceException("Hibernate dialect name " + jdbcDialectName + " not supported");
      }
    }
    return jdbcDialectName;
  }

  public static String autodetectJdbcDialect(DataSource datasource) {
    try {
        String jdbcURL = datasource.getConnection().getMetaData().getURL();
        return getHibernateDialectName(jdbcURL);

    } catch (SQLException e) {
      throw new PersistenceException("can not connect to databse", e);
    }
  }
}